#!/usr/bin/env python

"""
@author: Alessandro Facciorusso
"""

from gi.repository import Gtk
import os
import sys
import pkg_resources

from bumblebee_config.Settings import Settings

__version__ = pkg_resources.require("bumblebee-config-gtk")[0].version


def determine_path():
    try:
        root = __file__

        if os.path.islink(root):
            root = os.path.realpath(root)
        return os.path.dirname(os.path.abspath(root))
    except:
        raise SystemError("Could not find the __file__ var. Exiting.")
        sys.exit()


class BumblebeeConfig(object):

    def __init__(self):
        super(BumblebeeConfig, self).__init__()

        self.widgetList = []

        # widget, type, section, key
        # for radio only the first
        self.widgetsMap = [
            ["virtDispEntry", "entry", "bumblebeed", "VirtualDisplay"],
            ["keepUnusServerRadio1", "radio", "bumblebeed", "KeepUnusedXServer"],
            ["serverGroupEntry", "entry", "bumblebeed", "ServerGroup"],
            ["turnCardOffRadio1", "radio", "bumblebeed", "TurnCardOffAtExit"],
            ["noEcoModeRadio1", "radio", "bumblebeed", "NoEcoModeOverride"],
            ["driverComboBox", "cbbox", "bumblebeed", "Driver"],

            ["bridgeComboBox", "cbbox", "optirun", "Bridge"],
            ["vglTransportComboBox", "cbbox", "optirun", "VGLTransport"],
            ["allowFallbackToIgcRadio1", "radio", "optirun", "AllowFallbackToIGC"],

            ["kernelDriverNvidiaEntry", "entry", "driver-nvidia", "KernelDriver"],
            ["pmMethodNvidiaComboBox", "cbbox", "driver-nvidia", "PMMethod"],
            ["libraryPathEntry", "entry", "driver-nvidia", "LibraryPath"],
            ["xorgModulePathEntry", "entry", "driver-nvidia", "XorgModulePath"],
            ["xorgConfFileNvidiaEntry", "entry", "driver-nvidia", "XorgConfFile"],

            ["kernelDriverNouveauEntry", "entry", "driver-nouveau", "KernelDriver"],
            ["pmMethodNouveauComboBox", "cbbox", "driver-nouveau", "PMMethod"],
            ["xorgConfFileNouveauEntry", "entry", "driver-nouveau", "XorgConfFile"]
        ]

        self.settings = Settings()

        self.builder = Gtk.Builder()
        self.builder.add_from_file(os.path.join(determine_path(), "main_window.ui"))

        self.mainWindow = self.builder.get_object("window1")
        self.mainWindow.set_title("Bumblebee Configurator Gui " + __version__)

        self.builder.get_object("quitToolButton").connect("clicked", Gtk.main_quit)
        self.builder.get_object("saveToolButton").connect("clicked", self._updateAndSaveSettings)

        for name, widgetType, section, key in self.widgetsMap:
            self.widgetList.append([self.builder.get_object(name), widgetType, section, key])

        self._updateWidgets()

    def _updateAndSaveSettings(self, widget):
        msg = self.settings.saveSettings()

        dialog = Gtk.MessageDialog(self.mainWindow, 0, Gtk.MessageType.INFO,
                                   Gtk.ButtonsType.OK, "Bumblebee output")
        dialog.format_secondary_text(msg)
        dialog.run()
        dialog.destroy()
        print msg
        print "Settings saved."

    def _onRadioChanged(self, widget, data):
        (section, key) = data
        if widget.get_active():
            value = "true"
        else:
            value = "false"
        self.settings[section][key] = value

    def _onEntryChanged(self, widget, data):
        (section, key) = data
        self.settings[section][key] = widget.get_text()

    def _onCbboxChanged(self, widget, data):
        (section, key) = data
        self.settings[section][key] = widget.get_active_text().lower()

    def _updateWidgets(self):
        for widget, widgetType, section, key in self.widgetList:
            if widgetType == "entry":
                widget.set_text(self.settings[section][key])
                widget.connect("changed", self._onEntryChanged, [section, key])
            elif widgetType == "radio":
                group = widget.get_group()
                for el in group:
                    if self.settings[section][key].lower() == "true" and el == widget:
                        widget.set_active(True)
                        break
                    elif self.settings[section][key].lower() == "false" and el != widget:
                        el.set_active(True)
                        break

                widget.connect("toggled", self._onRadioChanged, [section, key])
            elif widgetType == "cbbox":
                model = widget.get_model()
                i = 0
                for el in model:
                    el1 = el[0]
                    if self.settings[section][key] == "":
                        widget.set_active(0)
                        break
                    else:
                        if self.settings[section][key] == el1.lower():
                            widget.set_active(i)
                            break

                    i += 1
                widget.connect("changed", self._onCbboxChanged, [section, key])

    def run(self):
        self.mainWindow.show_all()
        self.mainWindow.connect("delete-event", Gtk.main_quit)
        Gtk.main()
